package com.sun.java.util.jar.pack;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.zip.ZipEntry;
import java.io.FileInputStream;
import java.io.File;
import java.util.jar.JarOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.zip.CRC32;
import java.io.BufferedInputStream;

class NativeUnpack
{
    private long unpackerPtr;
    private BufferedInputStream in;
    private int _verbose;
    private long _byteCount;
    private int _segCount;
    private int _fileCount;
    private long _estByteLimit;
    private int _estSegLimit;
    private int _estFileLimit;
    private int _prevPercent;
    private final CRC32 _crc32;
    private byte[] _buf;
    private UnpackerImpl _p200;
    private PropMap _props;
    
    private static synchronized native void initIDs();
    
    private synchronized native long start(ByteBuffer p0, long p1);
    
    private synchronized native boolean getNextFile(Object[] p0);
    
    private synchronized native ByteBuffer getUnusedInput();
    
    private synchronized native long finish();
    
    protected synchronized native boolean setOption(String p0, String p1);
    
    protected synchronized native String getOption(String p0);
    
    NativeUnpack(UnpackerImpl p) {
        this._prevPercent = -1;
        this._crc32 = new CRC32();
        this._buf = new byte[16384];
        this._p200 = p;
        this._props = p.props;
        p._nunp = this;
    }
    
    private static Object currentInstance() {
        UnpackerImpl unpackerImpl = (UnpackerImpl)Utils.getTLGlobals();
        return (unpackerImpl == null) ? null : unpackerImpl._nunp;
    }
    
    private synchronized long getUnpackerPtr() {
        return this.unpackerPtr;
    }
    
    private long readInputFn(ByteBuffer byteBuffer, long lng) throws IOException {
        if (this.in == null) {
            return 0L;
        }
        long lng2 = (long)(byteBuffer.capacity() - byteBuffer.position());
        assert lng <= lng2;
        long lng3 = 0L;
        int i = 0;
        while (lng3 < lng) {
            ++i;
            int length = this._buf.length;
            if ((long)length > lng2 - lng3) {
                length = (int)(lng2 - lng3);
            }
            int read = this.in.read(this._buf, 0, length);
            if (read <= 0) {
                break;
            }
            lng3 += (long)read;
            assert lng3 <= lng2;
            byteBuffer.put(this._buf, 0, read);
        }
        if (this._verbose > 1) {
            Utils.log.fine("readInputFn(" + lng + "," + lng2 + ") => " + lng3 + " steps=" + i);
        }
        if (lng2 > 100L) {
            this._estByteLimit = this._byteCount + lng2;
        }
        else {
            this._estByteLimit = (this._byteCount + lng3) * 20L;
        }
        this._byteCount += lng3;
        this.updateProgress();
        return lng3;
    }
    
    private void updateProgress() {
        double n = (double)this._segCount;
        if (this._estByteLimit > 0L && this._byteCount > 0L) {
            n += (double)this._byteCount / (double)this._estByteLimit;
        }
        int n2 = (int)Math.round(100.0 * (0.33 * n / (double)Math.max(this._estSegLimit, 1) + 0.67 * (double)this._fileCount / (double)Math.max(this._estFileLimit, 1)));
        if (n2 > 100) {
            n2 = 100;
        }
        if (n2 > this._prevPercent) {
            this._prevPercent = n2;
            this._props.setInteger("unpack.progress", n2);
            if (this._verbose > 0) {
                Utils.log.info("progress = " + n2);
            }
        }
    }
    
    private void copyInOption(String s) {
        String property = this._props.getProperty(s);
        if (this._verbose > 0) {
            Utils.log.info("set " + s + "=" + property);
        }
        if (property != null && !this.setOption(s, property)) {
            Utils.log.warning("Invalid option " + s + "=" + property);
        }
    }
    
    void run(InputStream in, JarOutputStream jarOutputStream, ByteBuffer unusedInput) throws IOException {
        BufferedInputStream in2 = new BufferedInputStream(in);
        this.in = in2;
        this._verbose = this._props.getInteger("com.sun.java.util.jar.pack.verbose");
        int n = "keep".equals(this._props.getProperty("com.sun.java.util.jar.pack.unpack.modification.time", "0")) ? 0 : this._props.getTime("com.sun.java.util.jar.pack.unpack.modification.time");
        this.copyInOption("com.sun.java.util.jar.pack.verbose");
        this.copyInOption("unpack.deflate.hint");
        if (n == 0) {
            this.copyInOption("com.sun.java.util.jar.pack.unpack.modification.time");
        }
        this.updateProgress();
        while (true) {
            long start = this.start(unusedInput, 0L);
            long n2 = 0L;
            this._estByteLimit = n2;
            this._byteCount = n2;
            ++this._segCount;
            int n3 = (int)(start >>> 32);
            int n4 = (int)(start >>> 0);
            this._estSegLimit = this._segCount + n3;
            this._estFileLimit = (int)((double)(this._fileCount + n4) * (double)this._estSegLimit / (double)this._segCount);
            int[] array = { 0, 0, 0, 0 };
            Object[] array2 = { array, null, null, null };
            while (this.getNextFile(array2)) {
                this.writeEntry(jarOutputStream, (String)array2[1], (n != 0) ? ((long)n) : ((long)array[2]), ((long)array[0] << 32) + ((long)array[1] << 32 >>> 32), array[3] != 0, (ByteBuffer)array2[2], (ByteBuffer)array2[3]);
                ++this._fileCount;
                this.updateProgress();
            }
            unusedInput = this.getUnusedInput();
            long finish = this.finish();
            if (this._verbose > 0) {
                Utils.log.info("bytes consumed = " + finish);
            }
            if (unusedInput == null && !Utils.isPackMagic(Utils.readMagic(in2))) {
                break;
            }
            if (this._verbose > 0 && unusedInput != null) {
                Utils.log.info("unused input = " + unusedInput);
            }
            else {
                continue;
            }
        }
    }
    
    void run(InputStream inputStream, JarOutputStream jarOutputStream) throws IOException {
        this.run(inputStream, jarOutputStream, null);
    }
    
    void run(File file, JarOutputStream jarOutputStream) throws IOException {
        __Null _Null = null;
        try (final FileInputStream fileInputStream = new FileInputStream(file)) {
            this.run(fileInputStream, jarOutputStream, _Null);
        }
    }
    
    private void writeEntry(JarOutputStream jarOutputStream, String s, long n, long lng, boolean b, ByteBuffer byteBuffer, ByteBuffer byteBuffer2) throws IOException {
        int i = (int)lng;
        if ((long)i != lng) {
            throw new IOException("file too large: " + lng);
        }
        CRC32 crc32 = this._crc32;
        if (this._verbose > 1) {
            Utils.log.fine("Writing entry: " + s + " size=" + i + (b ? " deflated" : ""));
        }
        if (this._buf.length < i) {
            while (i < this._buf.length) {
                i <<= 1;
                if (i > 0) {
                    continue;
                }
                break;
            }
            this._buf = new byte[i];
        }
        assert this._buf.length >= i;
        int j = 0;
        if (byteBuffer != null) {
            int capacity = byteBuffer.capacity();
            byteBuffer.get(this._buf, j, capacity);
            j += capacity;
        }
        if (byteBuffer2 != null) {
            int capacity2 = byteBuffer2.capacity();
            byteBuffer2.get(this._buf, j, capacity2);
            j += capacity2;
        }
        while (j < i) {
            int read = this.in.read(this._buf, j, i - j);
            if (read <= 0) {
                throw new IOException("EOF at end of archive");
            }
            j += read;
        }
        ZipEntry ze = new ZipEntry(s);
        ze.setTime(n * 1000L);
        if (i == 0) {
            ze.setMethod(0);
            ze.setSize(0L);
            ze.setCrc(0L);
            ze.setCompressedSize(0L);
        }
        else if (!b) {
            ze.setMethod(0);
            ze.setSize((long)i);
            ze.setCompressedSize((long)i);
            crc32.reset();
            crc32.update(this._buf, 0, i);
            ze.setCrc(crc32.getValue());
        }
        else {
            ze.setMethod(8);
            ze.setSize((long)i);
        }
        jarOutputStream.putNextEntry(ze);
        if (i > 0) {
            jarOutputStream.write(this._buf, 0, i);
        }
        jarOutputStream.closeEntry();
        if (this._verbose > 0) {
            Utils.log.info("Writing " + Utils.zeString(ze));
        }
        return;
    }
    
    static {
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                System.loadLibrary("unpack");
                return null;
            }
        });
        initIDs();
    }
}
